#include <xc.h>

// CONFIG1
#pragma config FOSC = HS        // Oscillator Selection (HS Oscillator, High-speed crystal/resonator connected between OSC1 and OSC2 pins)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = ON        // Internal/External Switchover (Internal/External Switchover mode is enabled)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is enabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF       // PLL Enable (4x PLL enabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

//PICDEM 2 PLUS ??? ???
#define PICDEM2PLUS


//PIC MCU?? ???? ??? ?? ??
#define Fosc 20000000 			//?? 20MHz ?? ??
#define Fcy Fosc / 4 			// Fcy = Fosc/4
#define Tcy 1 / Fcy 			// 1/4MHz = 250ns

//PIC MCU? ?? ???(Fosc) 16MHz? ??, UART ?? ?-??? ?? 9600 bps ??
#define BAUD_RATE 9600
#define BRG_VALUE (Fosc/16 /BAUD_RATE - 1)

// LED ??? ?? I/O ?? ?????.
#ifdef PICDEM2PLUS
    #define  LED		PORTBbits.RB0 //D2 LED ??, ??? ?? ?? ??
#endif


//UART?? ??? ??? ??

//?? ?? ??
unsigned char rx_data;          //UART ?? ?? ??? ? ?? ??
const unsigned char ABCD[] = {"This is UART string Example"};
volatile unsigned char ADC_Result = 0;


//IO?? ??? ??
#ifdef PICDEM2PLUS
   void initial_port(void)
   {
        ANSELA = 0xFF; // ???? ??? ??
        TRISA = 0xFF; // ????


        ANSELB = 0x00; //??B? ANxx ?? ?? ???? ??
	//PORTB ?? LED ?? ?? ???
	TRISB = 0xFF; //PORTB RB0?? ???? ?? INT ??
	// 1 = ??, 0 = ??

        ANSELD = 0x00;
        TRISD = 0x00;
        PORTD = 0xFF;

	TRISCbits.TRISC7 = 1; //??? RC7(RX)? ???
	TRISCbits.TRISC6 = 0; //??? RC6(TX)? ???
   }
#endif


void ADC_Init(void)
{
    ADCON0 = 0x01; // Channel 0, Go/Done is as 0
    ADCON1 = 0x50; // LEFT,  V+ -> Vdd  // V- -> GND
}


//UART ?? ??? ??
void initial_uart(void)
{
	//?? ???? ?? : 9600 bps
	//SPBRG = BRG_VALUE; 	//define??? ???
        SPBRG = 128;
	//TXSTA ???? ?? ???
	TXEN = 1;	//TX ?? ?? On [0:Disable, 1:Enable]
	BRGH = 1;	//?? ????? ?? On [0:Disable, 1:Enable]
	TX9 = 0;	//9Bit ?? ?? ?? [0:8Bit, 1:9Bit]
    	SYNC = 0;	//??/??? ?? ??  [0:???, 1:??]
	//RCSTA ???? ?? ???
	SPEN = 1;	//????? RX, TX ? ??[0:Disable, 1:Enable]
	CREN =1;	//?? ?? ?? On [0:Disable, 1:Enable]
	RX9 = 0;	//9Bit ?? ?? ?? [0:8Bit, 1:9Bit]
	//UART ???? ???
	RCIF = 0; 	//?? ???? ???
	TXIF = 0; 	//?? ???? ???
  	RCIE = 0;	//?? ???? ?? ??[0:Disable, 1:Enable]
  	TXIE = 0;	//?? ???? ?? ??[0:Disable, 1:Enable]
}

//UART ?? ? ?? ?? ??
void uart_putch(unsigned char data)
{
  	TXREG = data;
	while(!TRMT);
}

//UART ?? ? ?? ?? ??
unsigned char uart_getch(void)
{
  	while(!RCIF);
  	RCIF = 0;
  	return RCREG;
}

//UART ?? ??? ?? ??
void uart_puts(const unsigned char* ptr_string)
{
	while(*ptr_string != 0 )
	{
		TXREG = *ptr_string++;
		while(!TRMT);
	}
}


void interrupt ISR(void)
{
    if(INTF&INTE)
    {
        INTF=0;
        PORTD= ~PORTD;
    }
}


int main(int argc, char** argv)
{
    //????? ???
    OSCCON = 0b01111000;	//16MHZ ?? ?? ??

    initial_port();             //LED? ?? ?? ??? ?? ??

    initial_uart();             //UART ?? ??? ??? ?? ??

    ADC_Init();
    uart_puts(ABCD);            //UART ?? ?? ??? ??

    INTEDG = 0; // ?????? ???? ??
    INTE = 1;
    GIE = 1;

    while(1)
    {
        rx_data = uart_getch(); //UART ?? ? ?? ??? ??? ??

        ADCON0bits.GO_nDONE=1;
        while(!ADCON0bits.GO_nDONE);
        ADC_Result = (ADRES >>8);
          //PORTD = (ADC_Result>>4);


        if(rx_data=='0')	//???????? ??? 0? ???
				//UART RX ??? 0x30 ? ???
	{

                LED = 0;	//LED ?
		uart_puts("LED Off !!");	//TX ??? ??
		uart_putch(0x0D);  		// ??????? ???,
		uart_putch(0x0A);		// ? ?? ??? ?? ??

                uart_putch((ADC_Result/100) + 0x30);	//TX ??? ??
                uart_putch(((ADC_Result%100)/10)+ 0x30);
                uart_putch((ADC_Result%10) + 0x30);
		uart_putch(0x0D);  		// ??????? ???,
		uart_putch(0x0A);		// ? ?? ??? ?? ??
	}
	else if(rx_data=='1')	//???????? ??? 1? ???
					//UART RX ??? 0x31 ? ???
	{
		LED = 1;
		uart_puts("LED On !!");	//TX ??? ??
		uart_putch(0x0D);  		// ??????? ???,
		uart_putch(0x0A);		// ? ?? ??? ?? ??
	}
    }
}
